/*Loads TGA files for their alpha channel, NeHe tutorial 32 */

#include <windows.h>
#include <stdio.h>
#include <gl\gl.h>								// Header File For The OpenGL32 Library
#include <gl\glu.h>

#pragma comment( lib, "opengl32.lib" )						// Search For OpenGL32.lib While Linking
#pragma comment( lib, "glu32.lib" )						// Search For GLu32.lib While Linking
#pragma comment( lib, "winmm.lib" )


typedef struct									// Create A Structure
{
	GLubyte	*imageData;							// Image Data (Up To 32 Bits)
	GLuint	bpp;								// Image Color Depth In Bits Per Pixel.
	GLuint	width;								// Image Width
	GLuint	height;								// Image Height
	GLuint	texID;								// Texture ID Used To Select A Texture
} TextureImage;	



TextureImage textures[2];

float GetHeight(float x, float z);


bool LoadTGA(TextureImage *texture, char *filename)				// Loads A TGA File Into Memory
{    
	GLubyte		TGAheader[12]={0,0,2,0,0,0,0,0,0,0,0,0};		// Uncompressed TGA Header
	GLubyte		TGAcompare[12];						// Used To Compare TGA Header
	GLubyte		header[6];						// First 6 Useful Bytes From The Header
	GLuint		bytesPerPixel;						// Holds Number Of Bytes Per Pixel Used In The TGA File
	GLuint		imageSize;						// Used To Store The Image Size When Setting Aside Ram
	GLuint		temp;							// Temporary Variable
	GLuint		type=GL_RGBA;						// Set The Default GL Mode To RBGA (32 BPP)

	FILE *file = fopen(filename, "rb");					// Open The TGA File

	if(	file==NULL ||							// Does File Even Exist?
		fread(TGAcompare,1,sizeof(TGAcompare),file)!=sizeof(TGAcompare) ||	// Are There 12 Bytes To Read?
		memcmp(TGAheader,TGAcompare,sizeof(TGAheader))!=0 ||		// Does The Header Match What We Want?
		fread(header,1,sizeof(header),file)!=sizeof(header))		// If So Read Next 6 Header Bytes
	{
		if (file == NULL)						// Does The File Even Exist? *Added Jim Strong*
			return FALSE;						// Return False
		else								// Otherwise
		{
			fclose(file);						// If Anything Failed, Close The File
			return FALSE;						// Return False
		}
	}

	texture->width  = header[1] * 256 + header[0];				// Determine The TGA Width	(highbyte*256+lowbyte)
	texture->height = header[3] * 256 + header[2];				// Determine The TGA Height	(highbyte*256+lowbyte)
    
 	if(	texture->width	<=0 ||						// Is The Width Less Than Or Equal To Zero
		texture->height	<=0 ||						// Is The Height Less Than Or Equal To Zero
		(header[4]!=24 && header[4]!=32))				// Is The TGA 24 or 32 Bit?
	{
		fclose(file);							// If Anything Failed, Close The File
		return FALSE;							// Return False
	}

	texture->bpp	= header[4];						// Grab The TGA's Bits Per Pixel (24 or 32)
	bytesPerPixel	= texture->bpp/8;					// Divide By 8 To Get The Bytes Per Pixel
	imageSize		= texture->width*texture->height*bytesPerPixel;	// Calculate The Memory Required For The TGA Data

	texture->imageData=(GLubyte *)malloc(imageSize);			// Reserve Memory To Hold The TGA Data

	if(	texture->imageData==NULL ||					// Does The Storage Memory Exist?
		fread(texture->imageData, 1, imageSize, file)!=imageSize)	// Does The Image Size Match The Memory Reserved?
	{
		if(texture->imageData!=NULL)					// Was Image Data Loaded
			free(texture->imageData);				// If So, Release The Image Data

		fclose(file);							// Close The File
		return FALSE;							// Return False
	}

	for(GLuint i=0; i<int(imageSize); i+=bytesPerPixel)			// Loop Through The Image Data
	{									// Swaps The 1st And 3rd Bytes ('R'ed and 'B'lue)
		temp=texture->imageData[i];					// Temporarily Store The Value At Image Data 'i'
		texture->imageData[i] = texture->imageData[i + 2];		// Set The 1st Byte To The Value Of The 3rd Byte
		texture->imageData[i + 2] = temp;				// Set The 3rd Byte To The Value In 'temp' (1st Byte Value)
	}

	fclose (file);								// Close The File

	// Build A Texture From The Data
	glGenTextures(1, &texture[0].texID);					// Generate OpenGL texture IDs

	glBindTexture(GL_TEXTURE_2D, texture[0].texID);				// Bind Our Texture
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);	// Linear Filtered
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);	// Linear Filtered
	
	if (texture[0].bpp==24)							// Was The TGA 24 Bits
	{
		type=GL_RGB;							// If So Set The 'type' To GL_RGB
	}

	glTexImage2D(GL_TEXTURE_2D, 0, type, texture[0].width, texture[0].height, 0, type, GL_UNSIGNED_BYTE, texture[0].imageData);

	return true;								// Texture Building Went Ok, Return True
}


void InitTGA(void)
{
	LoadTGA(&textures[0],"plant4.tga");
	glAlphaFunc(GL_GREATER,0.1f);						
	glEnable(GL_ALPHA_TEST);

}

void DrawPlants(void)
{
	glPushMatrix();

	
	glTranslatef(50,140,-850);
	glColor3f(1,1,1);

	
	
	glBindTexture(GL_TEXTURE_2D,textures[0].texID);
	
	glBegin(GL_QUADS);
		
		glTexCoord2f(0,0);
		glVertex3f(0,0,0);

		glTexCoord2f(1,0);
		glVertex3f(40,0,0);

		glTexCoord2f(1,1);
		glVertex3f(40,40,0);

		glTexCoord2f(0,1);
		glVertex3f(0,40,0);



		glTexCoord2f(0,0);
		glVertex3f(18.5f,0,20);
		
		glTexCoord2f(1,0);
		glVertex3f(18.5f,0,-20);
		
		glTexCoord2f(1,1);
		glVertex3f(18.5f,40,-20);
		
		glTexCoord2f(0,1);
		glVertex3f(18.5,40,20);

	
	glEnd();



	glPopMatrix();
	glPushMatrix();

	glTranslatef(1000,250,-2400);
		glBegin(GL_QUADS);
		
		glTexCoord2f(0,0);
		glVertex3f(0,0,0);

		glTexCoord2f(1,0);
		glVertex3f(40,0,0);

		glTexCoord2f(1,1);
		glVertex3f(40,40,0);

		glTexCoord2f(0,1);
		glVertex3f(0,40,0);



		glTexCoord2f(0,0);
		glVertex3f(18.5,0,20);
		
		glTexCoord2f(1,0);
		glVertex3f(18.5,0,-20);
		
		glTexCoord2f(1,1);
		glVertex3f(18.5,40,-20);
		
		glTexCoord2f(0,1);
		glVertex3f(18.5,40,20);

	
	glEnd();

	glPopMatrix();

	glPushMatrix();
	
	
	glTranslatef(1300,250,-2500);
		glBegin(GL_QUADS);
		
		glTexCoord2f(0,0);
		glVertex3f(0,0,0);

		glTexCoord2f(1,0);
		glVertex3f(40,0,0);

		glTexCoord2f(1,1);
		glVertex3f(40,60,0);

		glTexCoord2f(0,1);
		glVertex3f(0,60,0);



		glTexCoord2f(0,0);
		glVertex3f(18.5,0,20);
		
		glTexCoord2f(1,0);
		glVertex3f(18.5,0,-20);
		
		glTexCoord2f(1,1);
		glVertex3f(18.5,60,-20);
		
		glTexCoord2f(0,1);
		glVertex3f(18.5,60,20);

	
	glEnd();

	glPopMatrix();



	glPushMatrix();
		glTranslatef(900,250,-2700);
		glBegin(GL_QUADS);
		
		glTexCoord2f(0,0);
		glVertex3f(0,0,0);

		glTexCoord2f(1,0);
		glVertex3f(40,0,0);

		glTexCoord2f(1,1);
		glVertex3f(40,60,0);

		glTexCoord2f(0,1);
		glVertex3f(0,60,0);



		glTexCoord2f(0,0);
		glVertex3f(18.5,0,20);
		
		glTexCoord2f(1,0);
		glVertex3f(18.5,0,-20);
		
		glTexCoord2f(1,1);
		glVertex3f(18.5,60,-20);
		
		glTexCoord2f(0,1);
		glVertex3f(18.5,60,20);

	
	glEnd();
	glPopMatrix();

	glPushMatrix();
		glTranslatef(1100,250,-2800);
		glBegin(GL_QUADS);
		
		glTexCoord2f(0,0);
		glVertex3f(0,0,0);

		glTexCoord2f(1,0);
		glVertex3f(40,0,0);

		glTexCoord2f(1,1);
		glVertex3f(40,80,0);

		glTexCoord2f(0,1);
		glVertex3f(0,80,0);



		glTexCoord2f(0,0);
		glVertex3f(18.5,0,20);
		
		glTexCoord2f(1,0);
		glVertex3f(18.5,0,-20);
		
		glTexCoord2f(1,1);
		glVertex3f(18.5,80,-20);
		
		glTexCoord2f(0,1);
		glVertex3f(18.5,80,20);

	
	glEnd();
	
	glPopMatrix();

}


